@@ -1,14 +1,13 @@
 from functools import partial
-from typing import Any, Dict, Literal, Optional, TypeVar, Union
+from typing import Any, Dict, Literal, Optional, TypeVar
 
 import casadi as cs
-import numpy as npy
+import numpy as np
+import numpy.typing as npt
 
-from csnlp.core.constraints import HasConstraints
-from csnlp.core.parameters import HasParameters
-from csnlp.solutions import DMStruct, Solution, subsevalf
-from csnlp.util.data import dict2struct
-from csnlp.util.funcs import invoke
+from csnlp.core.solutions import Solution, subsevalf
+from csnlp.nlps.constraints import HasConstraints
+from csnlp.nlps.parameters import HasParameters
 
 T = TypeVar("T", cs.SX, cs.MX)
 
@@ -88,13 +87,12 @@ class HasObjective(HasParameters[T], HasConstraints[T]):
         self._solver = cs.nlpsol(f"nlpsol_{self.name}", "ipopt", nlp, opts)
         self._solver_opts = opts
 
-    def refresh_solver(self, *args: Any, **kwargs: Any) -> None:
+    def refresh_solver(self) -> None:
         """Refresh and resets the internal solver function (with the same options, if
         previously set)."""
         if self._solver is not None:
             self.init_solver(self._solver_opts)
 
-    @invoke(refresh_solver)
     def minimize(self, objective: T) -> None:
         """Sets the objective function to be minimized.
 
@@ -112,33 +110,37 @@ class HasObjective(HasParameters[T], HasConstraints[T]):
         if not objective.is_scalar():
             raise ValueError("Objective must be scalar.")
         self._f = objective
+        self.refresh_solver()
 
-    @invoke(refresh_solver)
     def parameter(self, *args, **kwargs):
-        return super().parameter(*args, **kwargs)
+        out = super().parameter(*args, **kwargs)
+        self.refresh_solver()
+        return out
 
-    @invoke(refresh_solver)
     def variable(self, *args, **kwargs):
-        return super().variable(*args, **kwargs)
+        out = super().variable(*args, **kwargs)
+        self.refresh_solver()
+        return out
 
-    @invoke(refresh_solver)
     def constraint(self, *args, **kwargs):
-        return super().constraint(*args, **kwargs)
+        out = super().constraint(*args, **kwargs)
+        self.refresh_solver()
+        return out
 
     def solve(
         self,
-        pars: Union[None, DMStruct, Dict[str, npy.ndarray]] = None,
-        vals0: Union[None, DMStruct, Dict[str, npy.ndarray]] = None,
+        pars: Optional[Dict[str, npt.ArrayLike]] = None,
+        vals0: Optional[Dict[str, npt.ArrayLike]] = None,
     ) -> Solution:
         """Solves the NLP optimization problem.
 
         Parameters
         ----------
-        pars : DMStruct, dict[str, array_like], optional
+        pars : dict[str, array_like], optional
             Dictionary or structure containing, for each parameter in the NLP scheme,
             the corresponding numerical value. Can be `None` if no parameters are
             present.
-        vals0 : DMStruct, dict[str, array_like], optional
+        vals0 : dict[str, array_like], optional
             Dictionary or structure containing, for each variable in the NLP scheme, the
             corresponding initial guess. By default, initial guesses are not passed to
             the solver.
@@ -171,7 +173,7 @@ class HasObjective(HasParameters[T], HasConstraints[T]):
             "p": p,
             "lbx": self._lbx,
             "ubx": self._ubx,
-            "lbg": npy.concatenate((self._lbg, self._lbh)),
+            "lbg": np.concatenate((self._lbg, self._lbh)),
             "ubg": 0,
         }
         if vals0 is not None:
@@ -184,13 +186,8 @@ class HasObjective(HasParameters[T], HasConstraints[T]):
         lam_g = sol["lam_g"][: self.ng, :]
         lam_h = sol["lam_g"][self.ng :, :]
 
-        vars_ = self.variables
-        if self._csXX is cs.SX:
-            vals = vars_(sol["x"])
-        else:
-            vals = dict2struct(
-                {name: subsevalf(var, self._x, sol["x"]) for name, var in vars_.items()}
-            )
+        vars = self.variables
+        vals = {name: subsevalf(var, self._x, sol["x"]) for name, var in vars.items()}
         old = cs.vertcat(
             self._p, self._x, self._lam_g, self._lam_h, self._lam_lbx, self._lam_ubx
         )
@@ -198,7 +195,7 @@ class HasObjective(HasParameters[T], HasConstraints[T]):
         get_value = partial(subsevalf, old=old, new=new)
         solution = Solution[T](
             f=float(sol["f"]),
-            vars=vars_,
+            vars=vars,
             vals=vals,
             stats=self._solver.stats().copy(),
             _get_value=get_value,
