@@ -4,10 +4,29 @@ from copy import deepcopy
 from functools import cached_property
 from inspect import getmembers
 from pickletools import optimize
-from typing import Any, Dict, Optional, TypeVar
+from typing import Any, Dict, Iterator, Optional, TypeVar
 from warnings import warn
 
-from csnlp.util.data import is_casadi_object
+
+def is_casadi_object(obj: Any) -> bool:
+    """Checks if the object belongs to the CasADi module.
+
+    See https://stackoverflow.com/a/52783240/19648688 for more details.
+
+    Parameters
+    ----------
+    obj : Any
+        Any type of object.
+
+    Returns
+    -------
+    bool
+        A flag that states whether the object belongs to CasADi or not.
+    """
+    if not hasattr(obj, "__module__"):
+        return False
+    module: str = obj.__module__.split(".")[0]
+    return module == "casadi"
 
 
 def is_pickleable(obj: Any) -> bool:
@@ -46,7 +65,7 @@ class SupportsDeepcopyAndPickle:
     _GETFULLSTATE: Optional[bool] = None
 
     @contextmanager
-    def pickleable(self) -> None:
+    def pickleable(self) -> Iterator[None]:
         """Context manager that makes the class pickleable by automatically removing
         unpickleable states (opposite of `fullstate`)."""
         self._GETFULLSTATE = False
@@ -54,7 +73,7 @@ class SupportsDeepcopyAndPickle:
         self._GETFULLSTATE = None
 
     @contextmanager
-    def fullstate(self) -> None:
+    def fullstate(self) -> Iterator[None]:
         """Context manager that makes the class return the full state without removing
         unpickleable states (opposite of `pickleable`)."""
         self._GETFULLSTATE = True
@@ -69,7 +88,7 @@ class SupportsDeepcopyAndPickle:
         invalidate_caches : bool, optional
             If `True`, methods decorated with `csnlp.util.funcs.invalidate_cache` are
             called to clear cached properties/lru caches in the copied instance.
-            Otherwise, caches in the copy are not invalidated.
+            Otherwise, caches in the copy are not invalidated. By default, `True`.
 
         Returns
         -------
