@@ -4,6 +4,7 @@ from typing import Optional, Union
 
 import casadi as cs
 import numpy as np
+import numpy.typing as npt
 from scipy.special import comb
 
 SQRT2 = sqrt(2)
@@ -142,7 +143,7 @@ def norm_ppf(
     return SQRT2 * scale * cs.erfinv(2 * p - 1) + loc
 
 
-def nchoosek(n: Union[int, np.ndarray], k: int) -> Union[int, np.ndarray]:
+def nchoosek(n: Union[int, npt.ArrayLike], k: int) -> Union[int, np.ndarray]:
     """Emulates the `nchoosek` function from Matlab. Returns the binomial
     coefficient, i.e.,  the number of combinations of `n` items taken `k` at a
     time. If `n` is an array, then it is flatten and all possible combinations
@@ -157,7 +158,7 @@ def nchoosek(n: Union[int, np.ndarray], k: int) -> Union[int, np.ndarray]:
 
     Returns
     -------
-    int or array_like
+    int or array
         Depending on the type of input `n`, the output is either the total
         number of combinations or the combinations in a matrix.
     """
@@ -168,7 +169,7 @@ def nchoosek(n: Union[int, np.ndarray], k: int) -> Union[int, np.ndarray]:
     )
 
 
-def monomial_powers(d: int, k: int) -> np.ndarray:
+def monomial_powers(d: int, k: int) -> npt.NDArray[np.int64]:
     """Computes the powers of all `d`-dimensional monomials of degree `k`.
 
     Parameters
@@ -180,7 +181,7 @@ def monomial_powers(d: int, k: int) -> np.ndarray:
 
     Returns
     -------
-    np.ndarray
+    array of ints
         An array containing in each row the power of each index in order to
         obtain the desired monomial of power `k`.
     """
@@ -188,7 +189,7 @@ def monomial_powers(d: int, k: int) -> np.ndarray:
     dividers = np.column_stack(
         (
             np.zeros((m, 1), dtype=int),
-            np.row_stack(nchoosek(np.arange(1, k + d), d - 1)),
+            np.row_stack(nchoosek(np.arange(1, k + d), d - 1)),  # type: ignore
             np.full((m, 1), k + d, dtype=int),
         )
     )
