@@ -1,12 +1,13 @@
-from typing import Dict, Optional, Tuple, TypeVar, Union
+from typing import Dict, Literal, Optional, Tuple, TypeVar, Union
 
 import casadi as cs
 import numpy as np
+import numpy.typing as npt
 
-from csnlp.solutions import Solution
-from csnlp.util.array import hohessian, hojacobian
-from csnlp.util.data import array2cs, cs2array
-from csnlp.util.funcs import cached_property, invalidate_cache
+from csnlp.core.cache import cached_property, invalidate_cache
+from csnlp.core.data import array2cs, cs2array
+from csnlp.core.derivatives import hohessian, hojacobian
+from csnlp.core.solutions import Solution
 from csnlp.wrappers.wrapper import Nlp, Wrapper
 
 T = TypeVar("T", cs.SX, cs.MX)
@@ -149,7 +150,7 @@ class NlpSensitivity(Wrapper[T]):
         }
 
     @property
-    def licq(self) -> Union[np.ndarray, T]:
+    def licq(self) -> T:
         """Gets the symbolic matrix for LICQ, defined as
         ```
                     LICQ = [ dgdx^T, dhdx^T ]^T
@@ -171,7 +172,11 @@ class NlpSensitivity(Wrapper[T]):
         self,
         expr: T = None,
         solution: Optional[Solution] = None,
-    ) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[T, T]]:
+    ) -> Union[
+        Tuple[T, T],
+        Tuple[cs.DM, cs.DM],
+        Tuple[npt.NDArray[np.double], npt.NDArray[np.double]],
+    ]:
         """Performs the (symbolic or numerical) sensitivity of the NLP w.r.t. its
         parametrization, according to [1].
 
@@ -189,9 +194,9 @@ class NlpSensitivity(Wrapper[T]):
 
         Returns
         -------
-        Union[np.ndarray, casadi.SX, casadi.MX]
-            The NLP parametric sensitivity in the form of an numerical array, if a
-            solution is passed, or a symbolic vector.
+        2-element tuple of casadi.SX, or MX, or DM, or arrays
+            The 1st and 2nd-order NLP parametric sensitivity in the form of an array/DM,
+            if a solution is passed, or a symbolic vector SX/MX.
 
         Raises
         ------
@@ -272,23 +277,41 @@ class NlpSensitivity(Wrapper[T]):
         return dzdp, d2zdp2
 
     @invalidate_cache(jacobians, hessians, hojacobians)
-    def parameter(self, *args, **kwargs):
-        return self.nlp.parameter(*args, **kwargs)
+    def parameter(self, name: str, shape: Tuple[int, int] = (1, 1)) -> T:
+        """See `Nlp.parameter` method."""
+        return self.nlp.parameter(name, shape)
 
     @invalidate_cache(lagrangian, kkt, jacobians, hessians, hojacobians)
-    def variable(self, *args, **kwargs):
-        return self.nlp.variable(*args, **kwargs)
+    def variable(
+        self,
+        name: str,
+        shape: Tuple[int, int] = (1, 1),
+        lb: Union[npt.ArrayLike, cs.DM] = -np.inf,
+        ub: Union[npt.ArrayLike, cs.DM] = +np.inf,
+    ) -> Tuple[T, T, T]:
+        """See `Nlp.variable` method."""
+        return self.nlp.variable(name, shape, lb, ub)
 
     @invalidate_cache(lagrangian, kkt, jacobians, hessians, hojacobians)
-    def constraint(self, *args, **kwargs):
-        return self.nlp.constraint(*args, **kwargs)
+    def constraint(
+        self,
+        name: str,
+        lhs: Union[T, np.ndarray, cs.DM],
+        op: Literal["==", ">=", "<="],
+        rhs: Union[T, np.ndarray, cs.DM],
+        soft: bool = False,
+        simplify: bool = True,
+    ) -> Tuple[T, ...]:
+        """See `Nlp.constraint` method."""
+        return self.nlp.constraint(name, lhs, op, rhs, soft, simplify)
 
     @invalidate_cache(lagrangian, kkt, jacobians, hessians, hojacobians)
-    def minimize(self, *args, **kwargs):
-        return self.nlp.minimize(*args, **kwargs)
+    def minimize(self, objective: T) -> None:
+        """See `Nlp.minimize` method."""
+        return self.nlp.minimize(objective)
 
     @property
-    def _y_idx(self) -> Tuple[T, Union[slice, np.ndarray]]:
+    def _y_idx(self) -> Tuple[T, Union[slice, npt.NDArray[np.int64]]]:
         """Internal utility to return all the primal-dual variables and indices that are
         associated to non-redundant entries in the kkt conditions."""
         if self.nlp._csXX is cs.SX:
