@@ -1,9 +1,10 @@
 from contextlib import contextmanager
-from typing import Any, Generic, Type, TypeVar
+from typing import Any, Generic, Iterator, List, Type, TypeVar, Union
 
 import casadi as cs
 
-from csnlp.nlp import Nlp
+from csnlp.core.solutions import Solution
+from csnlp.nlps.nlp import Nlp
 from csnlp.util.io import SupportsDeepcopyAndPickle
 
 T = TypeVar("T", cs.SX, cs.MX)
@@ -53,12 +54,12 @@ class Wrapper(SupportsDeepcopyAndPickle, Generic[T]):
         return self.nlp.is_wrapped(wrapper_type)
 
     @contextmanager
-    def fullstate(self) -> None:
+    def fullstate(self) -> Iterator[None]:
         with super().fullstate(), self.nlp.fullstate():
             yield
 
     @contextmanager
-    def pickleable(self) -> None:
+    def pickleable(self) -> Iterator[None]:
         with super().pickleable(), self.nlp.pickleable():
             yield
 
@@ -68,7 +69,9 @@ class Wrapper(SupportsDeepcopyAndPickle, Generic[T]):
             raise AttributeError(f"Accessing private attribute '{name}' is prohibited.")
         return getattr(self.nlp, name)
 
-    def __call__(self, *args: Any, **kwds: Any) -> Any:
+    def __call__(
+        self, *args: Any, **kwds: Any
+    ) -> Union[Solution[T], List[Solution[T]]]:
         return (self.solve_multi if self.nlp.is_multi else self.solve)(*args, **kwds)
 
     def __str__(self) -> str:
